﻿--[[
Name: LibAvionOptions-2.0
Revision: $REVISION$
Author(s): Sirow (EU-Gorgonnash)
Documentation: http://www.wowace.com/wiki/LibAvionOptions-2.0
SVN: http://svn.wowace.com/wowace/trunk/LibAvionOptions-2.0
Description: Supply common options for Avion-2.0 (eg. GuideFrame)
Dependencies: LibAvion-2.0, CallbackHandler-1.0
]]


-------------------------------------------------------------------------------
-- Library - Version Info & Dependencies
-------------------------------------------------------------------------------
-- Version
local MAJOR 		= "LibAvionOptions-2.0"
local MINOR 		= tonumber(("$REVISION$"):match("%d+")) or 0

-- Shamelessly copied from LibGraph-2.0 (search for resize.tga)
--	Search for just Addon\\ at the front since the interface part often gets trimmed
--	Do this before anything else, so if it errors, any existing loaded copy of LibGraph-2.0
--	doesn't get modified with a newer revision (this one)
local TextureDirectory
do
	local path = string.match(debugstack(1,1,0), "AddOns\\(.+)LibAvionOptions%-2%.0%.lua")
	
	if path then
		TextureDirectory = "Interface\\AddOns\\" .. path
	else
		--error(MAJOR.." cannot determine the folder it is located in because the path is too long and got truncated in the debugstack(1,1,0) function call")
		TextureDirectory = "Interface\\AddOns\\LibAvionOptions-2.0\\LibAvionOptions-2.0\\" 
	end
end

-- Dependencies
if not LibStub 				  			then error(MAJOR .. " requires LibStub.") end
if not LibStub("LibAvion-2.0")  		then error(MAJOR .. " requires LibAvion-2.0.") end
if not LibStub("CallbackHandler-1.0") 	then error(MAJOR .. " requires CallbackHandler-1.0.") end


-------------------------------------------------------------------------------
-- Library - Register
-------------------------------------------------------------------------------
-- Register
local LibAvionOpt 	= LibStub:NewLibrary(MAJOR, MINOR)
if not LibAvionOpt then return end

-- Embed CallbackHandler and LibSimpleTimer
LibAvionOpt.events	= LibAvionOpt.events or LibStub("CallbackHandler-1.0"):New(LibAvionOpt)


-------------------------------------------------------------------------------
-- Library - Variables
-------------------------------------------------------------------------------
-- Compatibility level for Ace3 and Rock1
local style = {
	["ACE3.0"] = {
		["numRange"] 	= "range",
		["select"] 		= "select",
		["values"] 		= "values",
		["toggle"] 		= "toggle",
		["string"]		= "input",
	},
	["ROCK1.0"] = {
		["numRange"] 	= "number",
		["select"] 		= "choice",
		["values"] 		= "choices",
		["toggle"] 		= "boolean",
		["string"]		= "string",
	},
}

-- Translation
local L 		= _G.LibAvionOpt__L

-- Set embeeding variables
LibAvionOpt.mixinTargets = LibAvionOpt.mixinTargets or {}
local mixins = {
	"BuildAnimationOptions",
	"HideGuideFrame",
	"ShowGuideFrame",
	"UpdateGuideFrame",
	"IsGuideFrameShown",
	
	"RegisterCallback",
	"UnregisterCallback",
	
	"guideCache",
}

-- Cache for animations/guides
LibAvionOpt.guideCache	= LibAvionOpt.guideCache 	or {}
LibAvionOpt.caches		= LibAvionOpt.caches 		or {
	guideFrames 		= {},
	guideCounter		= 0,
}
local guideCache		= LibAvionOpt.guideCache
local guideFrames		= LibAvionOpt.caches.guideFrames
local guideCounter		= LibAvionOpt.caches.guideCounter

-- Load LibAvion stuff
local updateInterval 		= _G.LibAvion__updateInterval
local animationFrames		= _G.LibAvion__caches.animationFrames
local animationCounter		= _G.LibAvion__caches.animationCounter
local GetAvailableFrame 	= _G.LibAvion__GetAvailableFrame
local InitFrame				= _G.LibAvion__InitFrame
local UpdateAnimation		= _G.LibAvion__UpdateAnimation


-------------------------------------------------------------------------------
-- Library - Guide Frame - Core
-------------------------------------------------------------------------------
-- Create a new guide frame
local function MakeGuideFrame()
	-- Basic frame
	local frame = CreateFrame("Frame", nil)
	frame:EnableMouse(true)
	frame:SetResizable(true)
	frame:SetMovable(true)
	frame:SetMinResize(22, 22)
	frame:SetFrameStrata("HIGH")
	frame:SetBackdropColor(1, 1, 1, 0)
	
	-- Mouseover highlight and config mode visualisation
	local texture = frame:CreateTexture(nil, "HIGHLIGHT")
	texture:SetAllPoints(frame)
	texture:SetTexture(0, 1, 1)
	texture:SetAlpha(0.3)

	-- Display some descriptive text
	local text = frame:CreateFontString(nil, "HIGHLIGHT")
	text:SetFontObject(GameFontHighlightSmall)
	text:SetPoint("CENTER", frame, "CENTER", 0, 0)
	text:SetText("ERROR")

	-- Resize mouse-drag image
	local resizeTexture = frame:CreateTexture(nil, "HIGHLIGHT")
	resizeTexture:SetHeight(16)
	resizeTexture:SetWidth(16)
	resizeTexture:SetTexture(TextureDirectory .. "resize.tga")
	resizeTexture:SetPoint("BOTTOMRIGHT", -2, 2)
	
	-- Add slider for rotation adjustment
	local slider = CreateFrame("Slider", nil, frame)
	slider:SetOrientation("HORIZONTAL")
	slider:SetWidth(200)
	slider:SetHeight(16)
	slider:SetPoint("BOTTOM", frame, "TOP", 0, 5)

	slider:SetThumbTexture("Interface\\Buttons\\UI-SliderBar-Button-Horizontal")
	slider:SetBackdrop({
		bgFile = "Interface\\Buttons\\UI-SliderBar-Background",
		edgeFile = "Interface\\Buttons\\UI-SliderBar-Border",
		tile = true,
		edgeSize = 8,
		tileSize = 8,
		insets = {
			left = 3,
			right = 3,
			top = 3,
			bottom= 3
		}
	})
	slider:SetBackdropColor(0, 1, 1, 1)
	slider:SetBackdropBorderColor(0, 1, 1, 1)
	slider:SetMinMaxValues(0, 359)
	slider:SetValue(0)
	slider:SetValueStep(1)
	
	-- Will open options menu for this animation
	local menuButton = CreateFrame("Button", nil, frame, "UIPanelButtonTemplate")
	menuButton:SetWidth(60)
	menuButton:SetHeight(25)
	menuButton:SetText(L["Menu"])
	menuButton:SetPoint("TOP", frame, "BOTTOM", 0, -5)
	
	-- Button to mirror allong x axis
	local mirrorX = CreateFrame("Button", nil, menuButton, "UIPanelButtonTemplate")
	mirrorX:SetWidth(60)
	mirrorX:SetHeight(25)
	mirrorX:SetText(L["MirrorX"])
	mirrorX:SetPoint("RIGHT", menuButton, "LEFT", 0, 0)
	
	-- Button to mirror allong y axis
	local mirrorY = CreateFrame("Button", nil, menuButton, "UIPanelButtonTemplate")
	mirrorY:SetWidth(60)
	mirrorY:SetHeight(25)
	mirrorY:SetText(L["MirrorY"])
	mirrorY:SetPoint("LEFT", menuButton, "RIGHT", 0, 0)
	
	-- Close Button
	local closeButton = CreateFrame("Button", nil, menuButton, "UIPanelCloseButton")
	closeButton:SetFrameLevel(5)
	closeButton:SetPoint("TOP", menuButton, "BOTTOM", 0, -5)
	
	-- Add everything to the cache
	guideFrames[guideCounter] = {
		inUse 			= true,
		frame 			= frame,
		texture 		= texture,
		text 			= text,
		resizeTexture	= resizeTexture,
		slider 			= slider,
		menuButton 		= menuButton,
		mirrorX 		= mirrorX,
		mirrorY 		= mirrorY,
		closeButton 	= closeButton,
		configMode		= 0
	}
	
	-- Increment number of frames and return new number
	guideCounter = guideCounter + 1
	return (guideCounter-1)
end


-- Get an unused frame or create a new one (only ID)
local function GetAvailableGuideFrameID()
	-- Is there any unused/invisible frame?
	for index, cache in pairs(guideFrames) do
		if (not cache.inUse) then
			-- Enable inUse and return index
			cache.inUse 		= true
			cache.configMode	= 0
			
			return index
		end
	end
	
	-- If not, create one!
	return MakeGuideFrame()
end


-- Get an unused frame or create a new one
local function GetAvailableGuideFrame()
	local frameID = GetAvailableGuideFrameID()
	return guideFrames[frameID], frameID
end


-- Will return scale of animation frame, calculated from the guideFrame
local function GetGuideFrameScale(guideObject, animationTable)
	local L1, L2 = sqrt(
		animationTable.size.width  * animationTable.size.width + 
		animationTable.size.height * animationTable.size.height
	),
	1.2 * sqrt(
		guideObject.frame:GetWidth()  * guideObject.frame:GetWidth() + 
		guideObject.frame:GetHeight() * guideObject.frame:GetHeight()
	)

	return (L2 / L1)
end


-- Update the panel to match the guide frame's location (but don't save to database until we're finished)
local function GetGuideFramePosition(guideObject)
	local x, y = guideObject.frame:GetCenter()
	x = (x - (UIParent:GetWidth()  * UIParent:GetEffectiveScale()) / 2) / UIParent:GetEffectiveScale()
	y = (y - (UIParent:GetHeight() * UIParent:GetEffectiveScale()) / 2) / UIParent:GetEffectiveScale()

	return x, y
end


-- Update postition of frame, based on position of guideFrame
local function MoveListener(self, animationID)
	-- Fetch required tables
	local animationTable 	= guideCache[animationID].animationTable
	local guideObject 		= guideFrames[guideCache[animationID].guideFrameID]
	local x, y 				= GetGuideFramePosition(guideObject)

	-- If not in animation mode, update size(w&h) (and position)
	if (guideObject.configMode == 0) then
		animationTable.position = {
			x = x,
			y = y
		}
	elseif (guideObject.configMode == 1) then
		if (animationTable.isAnimated.position) then
			animationTable.animation.position = {
				x = x,
				y = y
			}
		end
	end
	
	-- Update animation
	self:UpdateGuideFrame(animationID)
end


-- Update size of frame, based on size of guideFrame
local function ResizeListener(self, animationID)
	-- Fetch required tables
	local animationTable 	= guideCache[animationID].animationTable
	local guideObject 		= guideFrames[guideCache[animationID].guideFrameID]
	
	-- Update size (w&h)
	if (guideObject.configMode == 0) then
		animationTable.size = {
			width  = guideObject.frame:GetWidth()  * 1.2,
			height = guideObject.frame:GetHeight() * 1.2
		}
		
	-- Update scale
	elseif (guideObject.configMode == 1) then
		if (animationTable.isAnimated.scale) then
			animationTable.animation.scale = GetGuideFrameScale(guideObject, animationTable)
		end
	end
	
	-- Update animation
	self:UpdateGuideFrame(animationID)
end


-- Update rotation
local function SliderListener(self, animationID) 
	-- Fetch required tables
	local animationTable 	= guideCache[animationID].animationTable
	local guideObject 		= guideFrames[guideCache[animationID].guideFrameID]

	-- In start-value mode
	if (guideObject.configMode == 0) then
		animationTable.rotation = guideObject.slider:GetValue() / 180 * math.pi

	--In animation-value mode
	elseif (guideObject.configMode == 1) then
		animationTable.animation.rotation = guideObject.slider:GetValue() / 180 * math.pi
	end
	
	-- Update animation
	self:UpdateGuideFrame(animationID)
end


-- This is called each time mouse button is pressed (and hold) 
local function MouseDownListener(self, animationID)
	-- Fetch required tables
	local animationTable 	= guideCache[animationID].animationTable
	local guideObject 		= guideFrames[guideCache[animationID].guideFrameID]
		
	-- Right-click mean, change config mode between start and animate values
	if (arg1 == "RightButton") then
		-- Switch config mode
		guideObject.configMode = (guideObject.configMode + 1) % 2
		
		-- Adjust animation frame
		self:UpdateGuideFrame(animationID)
		
	-- On left-clicks, listen for mouse-dragging
	elseif (arg1 == "LeftButton") then
		-- Set vars to figure out if the mouse is in the resize area or not
		local screenX, screenY = GetCursorPosition()
		local panelX = guideObject.frame:GetRight()
		local panelY = guideObject.frame:GetBottom()

		-- Adjust for screen scale
		local scale = guideObject.frame:GetEffectiveScale()
		panelX = panelX * scale
		panelY = panelY * scale
		
		-- Set bounds for lower-right hand corner (inside resize texture)
		local check1 = screenX <= panelX + 14
		local check2 = screenX >= panelX - 14
		local check3 = screenY <= panelY + 14
		local check4 = screenY >= panelY - 14
		
		-- Start resizing
		if check1 and check2 and check3 and check4 then
			if (guideObject.configMode == 0 or animationTable.isAnimated.scale) then
				-- Attach an OnUpdate call which fires every repaint while we're resizing
				guideObject.frame:SetScript("OnUpdate", function() ResizeListener(self, animationID) end)
			
				-- System call
				guideObject.frame:StartSizing("BOTTOMRIGHT")
			end
			
		-- Start moving
		else
			if (guideObject.configMode == 0 or animationTable.isAnimated.position) then
				-- Attach an OnUpdate call which fires every repaint while we're moving
				guideObject.frame:SetScript("OnUpdate", function() MoveListener(self, animationID) end)
			
				-- System call
				guideObject.frame:StartMoving()
			end
		end
	end
end


-- Mouse has been released, set and save settings
local function MouseUpListener(self, animationID)
-- Fetch required tables
	local animationTable 	= guideCache[animationID].animationTable
	local guideObject 		= guideFrames[guideCache[animationID].guideFrameID]
	
	-- Remove the move/resize listeners
	guideObject.frame:SetScript("OnUpdate", nil)
	
	-- Nothing to update on mouse up when just config mode changed
	if (arg1 ~= "RightButton") then
		-- Stop moving or resizing
		guideObject.frame:StopMovingOrSizing()

		-- Update position once more
		MoveListener(self, animationID)
		ResizeListener(self, animationID)
	end
end


-------------------------------------------------------------------------------
-- Library - Options/GuideFrame - API
-------------------------------------------------------------------------------
--[[
######################################################
# :BuildAnimationOptions(tableSytle, animationID, animationTable, textureList, soundList, functionList)
# Notes:
#	Builds an AceConfig3 compatible optionstable for given animation.
#	Can easily be converted to other formations, such as RockConfig/Ace2
# Arguments:
#	tableSytle (string) - Wether to use ACE3 or Rock option style ("ACE3.0", "ROCK1.0")
#	animationID (string) - The unique identifier for this animation.
#	animationTable (table) - The table that defines this animation, see EOF of LibAvion-2.0.lua for more.
#	textureList (table) - List of available textures (key=PATH, value=NAME)
#	soundList (table) - List of available sounds (key=PATH, value=NAME)
#	functionList (table) - List of available fuctions (key=FUNCTION-POINTER, value=NAME)
# Returns:
#	Options table for editing animation with an AceConfig3 addon.
######################################################
]]--
function LibAvionOpt:BuildAnimationOptions(tableSytle, animationID, animationTable, textureList, soundList, functionList)
	-- Supported framework?
	if (not style[tableSytle]) then
		return {}
	end
	
	-- Return OptionsTable
	return {
		start = {
			order = 2,
			type = 'group',
			name = L["StartValues"],
			desc = L["StartValues_Desc"],
			args = { 
				color = {
					order = 1,
					type = 'color',
					name = L["ColorAlpha"],
					desc = L["ColorAlpha_Desc"],
					get = function()
						return animationTable.color.r, animationTable.color.g, animationTable.color.b, animationTable.alpha
					end,
					set = function(info, r, g, b, a)
						animationTable.color.r = r
						animationTable.color.g = g
						animationTable.color.b = b
						animationTable.alpha = a
						
						self:UpdateGuideFrame(animationID)
					end,
					hasAlpha = true
				},
				positionX = {
					order = 2,
					type = style[tableSytle]["numRange"],
					name = L["PositionX"],
					desc = L["PositionX_Desc"],
					get = function()
						return animationTable.position.x
					end,
					set = function(info, val)
						animationTable.position.x = val
						
						self:UpdateGuideFrame(animationID)
					end,
					min = -math.floor(GetScreenHeight() * 1 / UIParent:GetScale() / 2),
					max = math.floor(GetScreenHeight() * 1 / UIParent:GetScale() / 2),
					step = 1,
				},
				positionY = {
					order = 3,
					type = style[tableSytle]["numRange"],
					name = L["PositionY"],
					desc = L["PositionY_Desc"],
					get = function()
						return animationTable.position.y
					end,
					set = function(info, val)
						animationTable.position.y = val
						
						self:UpdateGuideFrame(animationID)
					end,
					min = -math.floor(GetScreenWidth() * 1 / UIParent:GetScale() / 2),
					max = math.floor(GetScreenWidth() * 1 / UIParent:GetScale() / 2),
					step = 1,
				},
				width = {
					order = 4,
					type = style[tableSytle]["numRange"],
					name = L["Width"],
					desc = L["Width_Desc"],
					get = function()
						return animationTable.size.width
					end,
					set = function(info, val)
						animationTable.size.width = val
						
						self:UpdateGuideFrame(animationID)
					end,
					min = 0,
					max = math.floor(GetScreenWidth() * 1 / UIParent:GetScale()),
					step = 1,
				},
				height = {
					order = 5,
					type = style[tableSytle]["numRange"],
					name = L["Height"],
					desc = L["Height_Desc"],
					get = function()
						return animationTable.size.height
					end,
					set = function(info, val)
						animationTable.size.height = val
						
						self:UpdateGuideFrame(animationID)
					end,
					min = 0,
					max =  math.floor(GetScreenHeight() * 1 / UIParent:GetScale()),
					step = 1,
				},
				rotation = {
					order = 6,
					type = style[tableSytle]["numRange"],
					name = L["Rotation"],
					desc = L["Rotation_Desc"],
					get = function()
						return animationTable.rotation / math.pi * 180
					end,
					set = function(info, val)
						animationTable.rotation = val * math.pi / 180
						
						self:UpdateGuideFrame(animationID)
					end,
					min = 0,
					max = 359,
					step = 1,
				},
			},
		},
		animation = {
			order = 3,
			type = 'group',
			name = L["AnimationValues"],
			desc = L["AnimationValues_Desc"],
			args = { 
				color = {
					order = 1,
					type = 'color',
					name = L["ColorAlpha"],
					desc = L["ColorAlpha_Desc"],
					get = function()
						return animationTable.animation.color.r, animationTable.animation.color.g, animationTable.animation.color.b
					end,
					set = function(info, r, g, b, a)
						animationTable.animation.color.r = r
						animationTable.animation.color.g = g
						animationTable.animation.color.b = b
						
						self:UpdateGuideFrame(animationID)
					end,
				},
				positionX = {
					order = 2,
					type = style[tableSytle]["numRange"],
					name = L["PositionX"],
					desc = L["PositionX_Desc"],
					get = function()
						return animationTable.animation.position.x
					end,
					set = function(info, val)
						animationTable.animation.position.x = val
						
						self:UpdateGuideFrame(animationID)
					end,
					min = -math.floor(GetScreenHeight() * 1 / UIParent:GetScale() / 2),
					max = math.floor(GetScreenHeight() * 1 / UIParent:GetScale() / 2),
					step = 1,
				},
				positionY = {
					order = 3,
					type = style[tableSytle]["numRange"],
					name = L["PositionY"],
					desc = L["PositionY_Desc"],
					get = function()
						return animationTable.animation.position.y
					end,
					set = function(info, val)
						animationTable.animation.position.y = val
						
						self:UpdateGuideFrame(animationID)
					end,
					min = -math.floor(GetScreenWidth() * 1 / UIParent:GetScale() / 2),
					max = math.floor(GetScreenWidth() * 1 / UIParent:GetScale() / 2),
					step = 1,
				},
				rotation = {
					order = 4,
					type = style[tableSytle]["numRange"],
					name = L["Rotation"],
					desc = L["Rotation_Desc"],
					get = function()
						return animationTable.animation.rotation / math.pi * 180
					end,
					set = function(info, val)
						animationTable.animation.rotation = val * math.pi / 180
						
						self:UpdateGuideFrame(animationID)
					end,
					min = 0,
					max = 359,
					step = 1,
				},
				scale = {
					order = 5,
					type = style[tableSytle]["numRange"],
					name = L["Scale"],
					desc = L["Scale_Desc"],
					get = function()
						return animationTable.animation.scale
					end,
					set = function(info, val)
						animationTable.animation.scale = val
						
						self:UpdateGuideFrame(animationID)
					end,
					min = 0.1,
					max = 10,
					step = 0.05,
				},
			},
		},
		animationFunc = {
			order = 4,
			type = 'group',
			name = L["AnimationFunc"],
			desc = L["AnimationFunc_Desc"],
			args = { 
				alpha = {
					order = 1,
					type = style[tableSytle]["select"],
					name = L["Alpha"],
					desc = L["Alpha_Desc"],
					[style[tableSytle]["values"]] = functionList,
					style = 'dropdown',
					get = function()
						return animationTable.animationFunc.alpha
					end,
					set = function(info, val)
						if (animationTable.animationFuncNames.alpha) then
							animationTable.animationFuncNames.alpha	= functionList[val]
						end
						animationTable.animationFunc.alpha = val
						
						self:UpdateGuideFrame(animationID)
					end,
				},
				color = {
					order = 2,
					type = style[tableSytle]["select"],
					name = L["Color"],
					desc = L["Color_Desc"],
					[style[tableSytle]["values"]] = functionList,
					style = 'dropdown',
					get = function()
						return animationTable.animationFunc.color
					end,
					set = function(info, val)
						if (animationTable.animationFuncNames.color) then
							animationTable.animationFuncNames.color	= functionList[val]
						end
						animationTable.animationFunc.color = val
						
						self:UpdateGuideFrame(animationID)
					end,
				}, 
				position = {
					order = 3,
					type = style[tableSytle]["select"],
					name = L["Position"],
					desc = L["Position_Desc"],
					[style[tableSytle]["values"]] = functionList,
					style = 'dropdown',
					get = function()
						return animationTable.animationFunc.position
					end,
					set = function(info, val)
						if (animationTable.animationFuncNames.position) then
							animationTable.animationFuncNames.position	= functionList[val]
						end
						animationTable.animationFunc.position = val
						
						self:UpdateGuideFrame(animationID)
					end,
				}, 
				scale = {
					order = 4,
					type = style[tableSytle]["select"],
					name = L["Scale"],
					desc = L["Scale_Desc"],
					[style[tableSytle]["values"]] = functionList,
					style = 'dropdown',
					get = function()
						return animationTable.animationFunc.scale
					end,
					set = function(info, val)
						if (animationTable.animationFuncNames.scale) then
							animationTable.animationFuncNames.scale	= functionList[val]
						end
						animationTable.animationFunc.scale = val
						
						self:UpdateGuideFrame(animationID)
					end,
				}, 
				rotation = {
					order = 5,
					type = style[tableSytle]["select"],
					name = L["Rotation"],
					desc = L["Rotation_Desc"],
					[style[tableSytle]["values"]] = functionList,
					style = 'dropdown',
					get = function()
						return animationTable.animationFunc.rotation
					end,
					set = function(info, val)
						if (animationTable.animationFuncNames.rotation) then
							animationTable.animationFuncNames.rotation	= functionList[val]
						end
						animationTable.animationFunc.rotation = val
						
						self:UpdateGuideFrame(animationID)
					end,
				}, 
			},
		},
		addRandom = {
			order = 5,
			type = 'group',
			name = L["AddRandom"],
			desc = L["AddRandom_Desc"],
			args = {
				color = {
					order = 1,
					type = style[tableSytle]["numRange"],
					name = L["Color"],
					desc = L["Color_Desc"],
					get = function()
						if (animationTable.addRandom.color) then
							return animationTable.addRandom.color
						else
							return 0
						end
					end,
					set = function(info, val)
						if (val == 0) then
							animationTable.addRandom.color = nil
						else
							animationTable.addRandom.color = val
						end
						
						self:UpdateGuideFrame(animationID)
					end,
					min = 0,
					max = 250,
					step = 5,
				},
				position = {
					order = 2,
					type = style[tableSytle]["numRange"],
					name = L["Position"],
					desc = L["Position_Desc"],
					get = function()
						if (animationTable.addRandom.position) then
							return animationTable.addRandom.position
						else
							return 0
						end
					end,
					set = function(info, val)
						if (val == 0) then
							animationTable.addRandom.position = nil
						else
							animationTable.addRandom.position = val
						end
						
						self:UpdateGuideFrame(animationID)
					end,
					min = 0,
					max = 250,
					step = 5,
				},
				scale = {
					order = 3,
					type = style[tableSytle]["numRange"],
					name = L["Scale"],
					desc = L["Scale_Desc"],
					get = function()
						if (animationTable.addRandom.scale) then
							return animationTable.addRandom.scale
						else
							return 0
						end
					end,
					set = function(info, val)
						if (val == 0) then
							animationTable.addRandom.scale = nil
						else
							animationTable.addRandom.scale = val
						end
						
						self:UpdateGuideFrame(animationID)
					end,
					min = 0,
					max = 250,
					step = 5,
				},
				rotation = {
					order = 4,
					type = style[tableSytle]["numRange"],
					name = L["Rotation"],
					desc = L["Rotation_Desc"],
					get = function()
						if (animationTable.addRandom.rotation) then
							return animationTable.addRandom.rotation
						else
							return 0
						end
					end,
					set = function(info, val)
						if (val == 0) then
							animationTable.addRandom.rotation = nil
						else
							animationTable.addRandom.rotation = val
						end
						
						self:UpdateGuideFrame(animationID)
					end,
					min = 0,
					max = 250,
					step = 5,
				},
			},
		},
		isAnimated = {
			order = 6,
			type = 'group',
			name = L["IsAnimated"],
			desc = L["IsAnimated_Desc"],
			args = {
				color = {
					order = 1,
					type = style[tableSytle]["toggle"],
					name = L["Color"],
					desc = L["Color_Desc"],
					get = function()
						return animationTable.isAnimated.color
					end,
					set = function()
						animationTable.isAnimated.color = not animationTable.isAnimated.color
						
						self:UpdateGuideFrame(animationID)
					end,
				},
				position = {
					order = 2,
					type = style[tableSytle]["toggle"],
					name = L["Position"],
					desc = L["Position_Desc"],
					get = function()
						return animationTable.isAnimated.position
					end,
					set = function()
						animationTable.isAnimated.position = not animationTable.isAnimated.position
						
						self:UpdateGuideFrame(animationID)
					end,
				},
				scale = {
					order = 3,
					type = style[tableSytle]["toggle"],
					name = L["Scale"],
					desc = L["Scale_Desc"],
					get = function()
						return animationTable.isAnimated.scale
					end,
					set = function()
						animationTable.isAnimated.scale = not animationTable.isAnimated.scale
						
						self:UpdateGuideFrame(animationID)
					end,
				},
				rotation = {
					order = 4,
					type = style[tableSytle]["toggle"],
					name = L["Rotation"],
					desc = L["Rotation_Desc"],
					get = function()
						return animationTable.isAnimated.rotation
					end,
					set = function()
						animationTable.isAnimated.rotation = not animationTable.isAnimated.rotation
						
						self:UpdateGuideFrame(animationID)
					end,
				},
			},
		},
		timing = {
			order = 7,
			type = 'group',
			name = L["Durations"],
			desc = L["Durations_Desc"],
			args = { 
				waitTime = {
					order = 1,
					type = style[tableSytle]["numRange"],
					name = L["WaitTime"],
					desc = L["WaitTime_Desc"],
					get = function()
						return animationTable.waitTime
					end,
					set = function(info, val)
						animationTable.waitTime = val
						
						self:UpdateGuideFrame(animationID)
					end,
					min = 0,
					max = 30,
					step = updateInterval,
				},
				animationTime = {
					order = 2,
					type = style[tableSytle]["numRange"],
					name = L["AnimationTime"],
					desc = L["AnimationTime_Desc"],
					get = function()
						return animationTable.animationTime
					end,
					set = function(info, val)
						animationTable.animationTime = val
						
						self:UpdateGuideFrame(animationID)
					end,
					min = 0,
					max = 30,
					step = updateInterval,
				},
				fadeTime = {
					order = 3,
					type = style[tableSytle]["numRange"],
					name = L["FadeTime"],
					desc = L["FadeTime_Desc"],
					get = function()
						return animationTable.fadeTime
					end,
					set = function(info, val)
						animationTable.fadeTime = val
						
						self:UpdateGuideFrame(animationID)
					end,
					min = 0,
					max = 30,
					step = updateInterval,
				},
			},
		},
		customTexture = {
			order = 8,
			type = style[tableSytle]["string"],
			name = L["CustomTexture"],
			desc = L["CustomTexture_Desc"],
			get = function()
				return animationTable.textureFile
			end,
			set = function(info, val)
				if (animationTable.textureFileName) then
					animationTable.textureFileName = "None"
				end
				animationTable.textureFile = val
				
				self:UpdateGuideFrame(animationID)
			end,
		},
		texture = {
			order = 9,
			type = style[tableSytle]["select"],
			name = L["Texture"],
			desc = L["Texture_Desc"],
			[style[tableSytle]["values"]] = textureList,
			style = 'dropdown',
			get = function()
				return animationTable.textureFile
			end,
			set = function(info, val)
				if (animationTable.textureFileName) then
					animationTable.textureFileName	= textureList[val]
				end
				animationTable.textureFile = val
				
				self:UpdateGuideFrame(animationID)
			end,
		},
		mirrorX = {
			order = 10,
			type = style[tableSytle]["toggle"],
			name = L["MirrorX"],
			desc = L["MirrorX_Desc"],
			get = function()
				return animationTable.mirror.x
			end,
			set = function()
				animationTable.mirror.x = not animationTable.mirror.x
				
				self:UpdateGuideFrame(animationID)
			end,
		},
		mirrorY = {
			order = 11,
			type = style[tableSytle]["toggle"],
			name = L["MirrorY"],
			desc = L["MirrorY_Desc"],
			get = function()
				return animationTable.mirror.y
			end,
			set = function()
				animationTable.mirror.y = not animationTable.mirror.y
				
				self:UpdateGuideFrame(animationID)
			end,
		},
		customSound = {
			order = 12,
			type = style[tableSytle]["string"],
			name = L["CustomSound"],
			desc = L["CustomSound_Desc"],
			get = function()
				return animationTable.soundFile
			end,
			set = function(info, val)
				if (animationTable.soundFileName) then
					animationTable.soundFileName = "None"
				end
				animationTable.soundFile = val
					
				PlaySoundFile(val)
			end,
		},
		sound = {
			order = 13,
			type = style[tableSytle]["select"],
			name = L["Sound"],
			desc = L["Sound_Desc"],
			[style[tableSytle]["values"]] = soundList,
			style = 'dropdown',
			get = function()
				return animationTable.soundFile
			end,
			set = function(info, val)
				if (animationTable.soundFileName) then
					animationTable.soundFileName = soundList[val]
				end
				animationTable.soundFile = val
					
				PlaySoundFile(val)
			end,
		},
		fadeMode = {
			order = 14,
			type = style[tableSytle]["select"],
			name = L["fadeMode"],
			desc = L["fadeMode_Desc"],
			[style[tableSytle]["values"]] = {
				[0] = L["JustFade"],
				[1] = L["AnimateAndFade"],
				[2] = L["InvertOnFade"],
			},
			get = function()
				return animationTable.fadeMode
			end,
			set = function(info, val)
				animationTable.fadeMode = val
				
				self:UpdateGuideFrame(animationID)
			end,
		},
		showWhileWait = {
			order = 15,
			type = style[tableSytle]["toggle"],
			name = L["ShowWhileWait"],
			desc = L["ShowWhileWait_Desc"],
			get = function()
				return animationTable.showWhileWait
			end,
			set = function()
				animationTable.showWhileWait = not animationTable.showWhileWait
				
				self:UpdateGuideFrame(animationID)
			end,
		},
		noEnd = {
			order = 16,
			type = style[tableSytle]["toggle"],
			name = L["NoEnd"],
			desc = L["NoEnd_Desc"],
			get = function()
				return animationTable.noEnd
			end,
			set = function()
				animationTable.noEnd = not animationTable.noEnd
				
				self:UpdateGuideFrame(animationID)
			end,
		},
		guideFrame = {
			order = 17,
			type = 'execute',
			name = L["GuideFrame"],
			desc = L["GuideFrame_Desc"],
			func = function()
				if (not self:IsGuideFrameShown(animationID)) then
					self:ShowGuideFrame(animationID, animationTable)
				else
					self:HideGuideFrame(animationID)
				end
			end
		},
	}
end


--[[
######################################################
# :ShowGuideFrame(animationID, animationTable)
# Notes:
#	Shows a setup-overlay for easier animation configuration.
# Arguments:
#	animationID (string) - The unique identifier for this animation.
#	animationTable (table) - The table that defines this animation, see EOF for more.
# Returns:
######################################################
]]--
function LibAvionOpt:ShowGuideFrame(animationID, animationTable)
	if (not self:IsGuideFrameShown(animationID)) then
		-- Fetch required frames
		local animationObject, animationFrameID = GetAvailableFrame()
		local guideObject, guideFrameID 		= GetAvailableGuideFrame()
		
		-- Save link to animation und guide frame that's beeing used
		guideCache[animationID] = {
			animationTable		= animationTable,
			animationFrameID	= animationFrameID,
			guideFrameID		= guideFrameID,
		}
		
		-- Save which triggerID triggerd this animation
		animationObject.triggerID 	= animationID
		guideObject.triggerID 		= animationID
		
		-- Init and Update frame
		guideObject.frame:Show()
		guideObject.texture:Show()
		guideObject.text:Show()
		guideObject.resizeTexture:Show()
		guideObject.slider:Show()
		guideObject.menuButton:Show()
		guideObject.mirrorX:Show()
		guideObject.mirrorY:Show()
		guideObject.closeButton:Show()
		self:UpdateGuideFrame(animationID)
		
		-- Init frame methods
		guideObject.slider:SetScript("OnValueChanged", 
			function() 
				SliderListener(self, animationID)
			end
		)
		guideObject.menuButton:SetScript("OnClick", 
			function() 
				self.events:Fire("LibAvion_MenuRequest", animationID, guideObject.frame) 
			end
		)
		guideObject.mirrorX:SetScript("OnClick",  			
			function()
				animationTable.mirror.x = not animationTable.mirror.x
				self:UpdateGuideFrame(animationID) 	
			end
		)
		guideObject.mirrorY:SetScript("OnClick", 			
			function()
				animationTable.mirror.y = not animationTable.mirror.y
				self:UpdateGuideFrame(animationID) 	
			end
		)
		guideObject.closeButton:SetScript("OnClick",
			function() 
				self:HideGuideFrame(animationID) 	
			end
		)
		guideObject.frame:SetScript("OnMouseDown",		
			function() 
				MouseDownListener(self, animationID) 
			end
		)
		guideObject.frame:SetScript("OnMouseUp",
			function() 
				MouseUpListener(self, animationID) 	
			end
		)
	end
end


--[[
######################################################
# :HideGuideFrame(animationID)
# Notes:
#	Hide setup-overlay again.
# Arguments:
#	animationID (string) - The unique identifier for this animation.
# Returns:
######################################################
]]--
function LibAvionOpt:HideGuideFrame(animationID)
	-- Create frame-memory
	if (self:IsGuideFrameShown(animationID)) then
		-- Fetch required tables
		local animationObject 	= animationFrames[guideCache[animationID].animationFrameID]
		local guideObject 		= guideFrames[guideCache[animationID].guideFrameID]
		
		-- Hide frame and its not inUse anymore...
		animationObject.frame:Hide()
		animationObject.texture:SetTexture("Interface\\Icons\\Temp.blp")
		animationObject.inUse = false
		
		-- Clean up
		guideObject.text:SetText("")
		guideObject.inUse = false
		
		-- Hide everything
		guideObject.frame:Hide()
		guideObject.texture:Hide()
		guideObject.text:Hide()
		guideObject.resizeTexture:Hide()
		guideObject.slider:Hide()
		guideObject.menuButton:Hide()
		guideObject.mirrorX:Hide()
		guideObject.mirrorY:Hide()
		guideObject.closeButton:Hide()
		
		
		-- Stop script and clear mem (or atleast give doggy GC something to eat)
		guideObject.slider:SetScript(		"OnValueChanged",	nil)
		guideObject.menuButton:SetScript(	"OnClick", 			nil)
		guideObject.mirrorX:SetScript(		"OnClick", 			nil)
		guideObject.mirrorY:SetScript(		"OnClick", 			nil)
		guideObject.closeButton:SetScript(	"OnClick", 			nil)
		guideObject.frame:SetScript(		"OnMouseDown",		nil)
		guideObject.frame:SetScript(		"OnMouseUp",		nil)
		
		-- Delete frame from guide cache
		guideCache[animationID].animationTable	= nil
		guideCache[animationID]					= nil
	end
end


--[[
######################################################
# :UpdateGuideFrame(animationID)
# Notes:
#	Will (force) the amimation to be updated according to updated animationTable.
# Arguments:
#	animationID (string) - The unique identifier for this animation.
# Returns:
######################################################
]]--
function LibAvionOpt:UpdateGuideFrame(animationID)
	-- Make sure frame is shown
	if (self:IsGuideFrameShown(animationID)) then
		-- Shorten things
		local animationTable 	= guideCache[animationID].animationTable
		local animationObject 	= animationFrames[guideCache[animationID].animationFrameID]
		local guideObject 		= guideFrames[guideCache[animationID].guideFrameID]
		
		-- Currently in start-value mode
		if (guideObject.configMode == 0) then
			-- Update background color
			guideObject.texture:SetTexture(0, 1, 1)
			guideObject.slider:SetBackdropColor(0, 1, 1, 1)
			guideObject.slider:SetBackdropBorderColor(0, 1, 1, 1)
			guideObject.text:SetText(L["GuideFrameMode0"])
			
			-- Reset position
			guideObject.frame:ClearAllPoints()
			guideObject.frame:SetPoint("CENTER", UIParent, "CENTER", 
				animationTable.position.x * UIParent:GetEffectiveScale(), 
				animationTable.position.y * UIParent:GetEffectiveScale()
			)
			guideObject.frame:SetWidth(animationTable.size.width   / 1.2)
			guideObject.frame:SetHeight(animationTable.size.height / 1.2)
			guideObject.slider:SetValue(animationTable.rotation / math.pi * 180)
			
			-- Set animation to its beginning
			animationObject.elapsedTimes = 0
		
		-- Currently in animate value mode
		elseif (guideObject.configMode == 1) then
			-- Update background color
			guideObject.texture:SetTexture(1, 1, 0)
			guideObject.slider:SetBackdropColor(1, 1, 0, 1)
			guideObject.slider:SetBackdropBorderColor(1, 1, 0, 1)
			guideObject.text:SetText(L["GuideFrameMode1"])
			
			-- Reset position
			guideObject.frame:ClearAllPoints()
			if (animationTable.isAnimated.position) then
				guideObject.frame:SetPoint("CENTER", UIParent, "CENTER", 
					animationTable.animation.position.x * UIParent:GetEffectiveScale(), 
					animationTable.animation.position.y * UIParent:GetEffectiveScale()
				)
			else
				guideObject.frame:SetPoint("CENTER", UIParent, "CENTER", 
					animationTable.position.x * UIParent:GetEffectiveScale(), 
					animationTable.position.y * UIParent:GetEffectiveScale()
				)
			end
			if (animationTable.isAnimated.scale) then
				guideObject.frame:SetWidth(animationTable.animation.scale  * animationTable.size.width  / 1.2)
				guideObject.frame:SetHeight(animationTable.animation.scale * animationTable.size.height / 1.2)
			else
				guideObject.frame:SetWidth(animationTable.size.width   / 1.2)
				guideObject.frame:SetHeight(animationTable.size.height / 1.2)
			end
			guideObject.slider:SetValue(animationTable.animation.rotation / math.pi * 180)
			
			-- Set animation to its end
			animationObject.elapsedTimes = animationTable.animationTime / updateInterval
		end
		
		-- Reset Frame
		InitFrame(animationObject, animationTable)
		
		-- Update animation
		UpdateAnimation(animationObject, animationTable, true, false)
		
		-- Reset timer (because it was updated in UpdateAnimation(...))
		animationObject.elapsedTimes = animationObject.elapsedTimes - 1
	end
end


--[[
######################################################
# :IsGuideFrameShown(animationID)
# Notes:
#	Check if there is allready an setup-overlay assiciated with this identifier.
# Arguments:
#	animationID (string) - The unique identifier for this animation.
# Returns:
#	True if there is allready one, else false.
######################################################
]]--
function LibAvionOpt:IsGuideFrameShown(animationID)
	if (guideCache and guideCache[animationID]) then
		return true
	else
		return false
	end
end


--[[
######################################################
# :ConvertAnimationTable(animationTable, textureList, soundList, functionList)
# Notes:
#	Will update the animation table, more precisly, it will set all entrys of animationFunc to
#	its corresponding entry (defined by animationFuncNames entrys) in functionList.
#	You should use this method to initialize your resently loaded animation tables, since functions
#	won't be saved by wow. Just store the functions string identifier in animationFuncNames
#	and use this methode to restore the animation table. (Some goes for texture and sound)
# Arguments:
#	animationTable (table) - The table that defines this animation, see EOF for more.
#	textureList (table) - List of available textures (key=PATH, value=NAME)
#	soundList (table) - List of available sounds (key=PATH, value=NAME)
#	functionList (table) - List of available fuctions (key=FUNCTION-POINTER, value=NAME)
# Returns:
######################################################
]]--
function LibAvionOpt:ConvertAnimationTable(animationTable, textureList, soundList, functionList)
	-- Update texture
	if (animationTable.textureFileName) then
		-- Scan through list, because value=name ...
		for path, name in pairs(textureList) do
			-- Alpha
			if (animationTable.textureFileName == name) then
				animationTable.textureFile = path
			end
			
			break
		end
	end
	
	-- Update sound
	if (animationTable.soundFileName) then
		-- Scan through list, because value=name ...
		for path, name in pairs(soundList) do
			-- Alpha
			if (animationTable.soundFileName == name) then
				animationTable.soundFile = path
			end
			
			break
		end
	end
	
	-- Update function pointers
	if (animationTable.animationFuncNames) then
		-- Scan through list, because value=name ...
		for funcPointer, funcName in pairs(functionList) do
			-- Alpha
			if (animationTable.animationFuncNames.alpha == funcName) then
				animationTable.animationFunc.alpha = funcPointer
			end
			
			-- Color
			if (animationTable.animationFuncNames.color == funcName) then
				animationTable.animationFunc.color = funcPointer
			end
			
			-- Position
			if (animationTable.animationFuncNames.position == funcName) then
				animationTable.animationFunc.position = funcPointer
			end
			
			-- Scale
			if (animationTable.animationFuncNames.scale == funcName) then
				animationTable.animationFunc.scale = funcPointer
			end
			
			-- Rotation
			if (animationTable.animationFuncNames.rotation == funcName) then
				animationTable.animationFunc.rotation = funcPointer
			end
		end
	end
end


--[[
######################################################
# :Embed(target)
# Notes:
#	Embeds methods listed in local <mixins> into target table.
# Arguments:
#	target (table) - The table on which to export methods onto.
# Returns:
#	The table provided, after embedding.
######################################################
]]--
function LibAvionOpt:Embed(target)
	LibAvionOpt.mixinTargets[target] = true
	
	for _, v in pairs(mixins) do
		target[v] = LibAvionOpt[v]
	end
	
	return target
end


-- Reembed (eg. lib was updated)
for addon in pairs(LibAvionOpt.mixinTargets) do
	LibAvionOpt:Embed(addon)
end


-------------------------------------------------------------------------------
-- Dokumentation - Variables
-------------------------------------------------------------------------------
--[[
VAR: guideCache 
TYPE: Table
DESC: Stores all currently active (shown) guide-frames, more speficly the unique key of the
frame (in animationFrames), mostly refered to as the FrameID (or animationFrameID) 
and the unique key of the frame (in guideFrames), mostly refered to as the  
guideFrameID that was assossiated to this animation. The frame thats beeing animated. 
Also saves (a link to)  the table that's used to define the animation.
This table is accessed by the unique key, that was used to show the guide-frame, namely animationID.


VAR: guideFrames
DESC: Table that contains a pool of created/available frame objects to be used for visualising
an animations option. It has "guideFramesCounter" number of entrys and 
the unique key that is used to access it is called guideFrameID.
Each entry inhabits the following entrys:
	inUse 		- Say if the frame is currently beeing used for visualising an animation
	frame 		- the actuell fram object that can be shown or hidden
	texture 		- the texture object
	text 			- Descriptive text
	resizeTexture	- Indicator for resizing the frame
	slider 		- Slider for rotation
	menuButton 	- Button to request a menu
	mirrorX 		- Button to mirror animation on the x axis
	mirrorY 		- Button to mirror animation on the y axis
	closeButton 	- Button to close options
	configMode	- Which values to edit (0 = StartValues; 1 = AnimationValues)


VAR: guideFramesCounter
TYPE: Number
DESC: Its just #guideFramesCounter, nothing more, nothing less


EOF]]